/*
  Copyright 2019 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.share

import android.content.Context
import android.content.Intent
import android.net.Uri
import com.google.gson.JsonObject
import com.kakao.sdk.common.KakaoSdk
import com.kakao.sdk.common.model.ApplicationInfo
import com.kakao.sdk.common.model.ClientError
import com.kakao.sdk.common.model.ClientErrorCause
import com.kakao.sdk.common.model.ContextInfo
import com.kakao.sdk.common.util.IntentResolveClient
import com.kakao.sdk.common.util.KakaoJson
import com.kakao.sdk.common.util.SdkLog
import com.kakao.sdk.share.model.KakaoTalkSharingAttachment
import com.kakao.sdk.share.model.SharingResult
import com.kakao.sdk.share.model.ValidationResult

/**
 * @suppress
 */
class KakaoTalkShareIntentClient(
    val contextInfo: ContextInfo = KakaoSdk.applicationContextInfo,
    val applicationInfo: ApplicationInfo = KakaoSdk.applicationContextInfo,
    val intentResolveClient: IntentResolveClient = IntentResolveClient.instance
) {
    @JvmOverloads
    fun sharingResultFromResponse(
        context: Context,
        response: ValidationResult,
        serverCallbackArgs: Map<String, String>?,
        appKey: String = applicationInfo.appKey,
        appVer: String = contextInfo.appVer
    ): SharingResult {
        val attachmentSize = attachmentSize(appKey, response, serverCallbackArgs)
        if (attachmentSize > Constants.TALK_SHARE_URI_LIMIT) {
            throw ClientError(
                ClientErrorCause.BadParameter,
                "KakaoTalk Share intent size is $attachmentSize bytes. It should be less than ${Constants.TALK_SHARE_URI_LIMIT} bytes."
            )
        }
        val sharingUri = sharingUriBuilder()
            .appendQueryParameter(Constants.LINKVER, Constants.LINKVER_40)
            .appendQueryParameter(Constants.APP_KEY, appKey)
            .appendQueryParameter(Constants.APP_VER, appVer)
            .appendQueryParameter(Constants.TEMPLATE_ID, response.templateId.toString())
            .appendQueryParameter(Constants.TEMPLATE_ARGS, response.templateArgs.toString())
            .appendQueryParameter(Constants.TEMPLATE_JSON, response.templateMsg.toString())
            .appendQueryParameter(
                Constants.EXTRAS,
                extrasWithServerCallbacks(contextInfo.extras, serverCallbackArgs).toString()
            )
            .build()
        SdkLog.i(sharingUri)
        val talkShareIntent = Intent(Intent.ACTION_SEND, sharingUri)
            .addFlags(Intent.FLAG_ACTIVITY_NEW_TASK or Intent.FLAG_ACTIVITY_CLEAR_TOP)
        return SharingResult(
            intentResolveClient.resolveTalkIntent(context, talkShareIntent) ?: throw ClientError(
                ClientErrorCause.NotSupported,
                "Kakaotalk not installed"
            ),
            // TODO: 데이터에 String타입 아닌 값 등 이상 있을때 어떻게 동작하는지 확인 (minor)
            KakaoJson.fromJson(response.warningMsg.toString(), Map::class.java),
            KakaoJson.fromJson(response.argumentMsg.toString(), Map::class.java)
        )
    }

    fun isKakaoTalkSharingAvailable(context: Context): Boolean =
        intentResolveClient.resolveTalkIntent(
            context,
            Intent(Intent.ACTION_VIEW, sharingUriBuilder().build())
        ) != null

    private fun sharingUriBuilder() =
        Uri.Builder().scheme(Constants.LINK_SCHEME).authority(Constants.TALK_SHARE_AUTHORITY)

    private fun attachmentSize(
        appKey: String,
        response: ValidationResult,
        serverCallbackArgs: Map<String, String>?
    ): Int {
        val attachment = KakaoTalkSharingAttachment(
            ak = appKey,
            P = response.templateMsg["P"].asJsonObject,
            C = response.templateMsg["C"].asJsonObject,
            ti = response.templateId,
            ta = response.templateArgs,
            extras = extrasWithServerCallbacks(contextInfo.extras, serverCallbackArgs)
        )
        return KakaoJson.toJson(attachment).length
    }

    private fun extrasWithServerCallbacks(
        extras: JsonObject,
        serverCallbackArgs: Map<String, String>?
    ): JsonObject {
        val clone = extras.deepCopy()
        if (serverCallbackArgs == null) return clone
        clone.addProperty(Constants.LCBA, KakaoJson.toJson(serverCallbackArgs))
        return clone
    }

    companion object {
        val instance by lazy { KakaoTalkShareIntentClient() }
    }
}