/*
  Copyright 2020 Kakao Corp.

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
 */
package com.kakao.sdk.user

import com.google.gson.JsonObject
import com.kakao.sdk.auth.TokenManageable
import com.kakao.sdk.auth.TokenManagerProvider
import com.kakao.sdk.auth.model.OAuthToken
import com.kakao.sdk.common.util.KakaoJson
import com.kakao.sdk.common.util.Utility
import com.kakao.sdk.network.ApiFactory
import com.kakao.sdk.user.model.User
import okhttp3.OkHttpClient
import okhttp3.mockwebserver.MockResponse
import okhttp3.mockwebserver.MockWebServer
import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.Assertions
import org.junit.jupiter.api.BeforeEach
import org.junit.jupiter.api.Test
import java.util.concurrent.CompletableFuture

class UserApiTest {
    private lateinit var server: MockWebServer
    private lateinit var api: UserApi
    private lateinit var client: UserApiClient

    @BeforeEach
    fun setup() {
        server = MockWebServer()
        api = ApiFactory.withClientAndAdapter(
            server.url("/").toString(),
            OkHttpClient.Builder()
        ).create(UserApi::class.java)
        client = UserApiClient(api, TokenManagerProvider(object : TokenManageable {
            override fun getToken(): OAuthToken? {
                return null
            }

            override fun setToken(token: OAuthToken) {}
            override fun clear() {}
        }))
    }

    @AfterEach
    fun cleanup() {
        server.shutdown()
    }

    @Test
    fun testMe() {
        val body = Utility.getJson("json/users/only_email.json")
        val response = MockResponse().setResponseCode(200).setBody(body)

        server.enqueue(response)

        val future = CompletableFuture<User>()

        client.me { user, error ->
            if (error != null) {
                future.completeExceptionally(error)
            } else {
                future.complete(user)
            }
        }

        Assertions.assertEquals("GET", server.takeRequest().method)

        val expected = KakaoJson.fromJson<JsonObject>(body, JsonObject::class.java)
        val result = future.get()
        println(result)

        Assertions.assertEquals(expected["id"].asLong, result.id)

        Assertions.assertEquals(
            expected["properties"].toString(),
            KakaoJson.toJson(result.properties)
        )     // TODO: JsonObject와 Map의 비교 더 좋은 방법 찾자
//        Assertions.assertEquals(dateFormat.parse(expected["connected_at"].asString), result.connectedAt)
//        Assertions.assertEquals(dateFormat.parse(expected["synched_at"].asString), result.synchedAt)
//        Assertions.assertEquals(expected["group_user_token"], result.groupUserToken)
    }
}